/*
 * cleaner.cc
 *
 *  Created on: 05.02.2011
 *      Author: ed
 */

#include "debug.h"
#include "meta.h"

#include "cleaner.h"

#include "fileitem.h"
#include "acfg.h"
#include "caddrinfo.h"
#include "tcpconnect.h"

#include <limits>
#include <cstring>
using namespace MYSTD;

#define TERM_VAL (time_t(-1))

cleaner g_victor;

cleaner::cleaner() : m_hint(END_OF_TIME), m_thr(0)
{
}

cleaner::~cleaner()
{
}

void cleaner::WorkLoop()
{
	LOGSTART("cleaner::WorkLoop");

	for(time_t nextRunTime(m_hint), knownHint(m_hint);;)
	{
		{
			setLockGuard;

			if(m_hint == TERM_VAL)
				return;

			if(m_hint != knownHint)
				nextRunTime=m_hint;

			wait_until(nextRunTime, 135);

			if(m_hint == TERM_VAL)
				return;

			knownHint = m_hint; // doing some work now, remember that state
		}
		nextRunTime = fileitem::DoDelayedUnregAndCheck();
		ldbg("nextRunTime now: " << nextRunTime);

		nextRunTime = min(acfg::ExecutePostponed(), nextRunTime);
		ldbg("nextRunTime now: " << nextRunTime);

		nextRunTime = min(CAddrInfo::ExpireCache(), nextRunTime);
		ldbg("nextRunTime now: " << nextRunTime);

		nextRunTime = min(tcpconnect::ExpireCache(), nextRunTime);
		ldbg("nextRunTime now: " << nextRunTime);
	};
}

inline void * CleanerThreadAction(void *pVoid)
{
	static_cast<cleaner*>(pVoid)->WorkLoop();
	return NULL;
}

void cleaner::RunLatestAt(time_t when)
{
	setLockGuard;

	if(m_hint == TERM_VAL)
		return; // cannot, shutting down

	m_hint = when;
	if(m_thr == 0)
		pthread_create(&m_thr, NULL, CleanerThreadAction, (void *)this);
	else
		notifyAll();
}

void cleaner::Stop()
{
	{
		setLockGuard;

		if(!m_thr)
			return;

		m_hint=TERM_VAL;
		notifyAll();
	}
    pthread_join(m_thr, NULL);

    setLockGuard;
    m_thr = 0;
    m_hint=END_OF_TIME;
}
